@@ -222,7 +222,7 @@ public class FileUtils {
           String rootPath = new File(fileToAdd.getParentFile().getCanonicalFile().getPath() + File.separator + fileToAdd.getCanonicalFile().getName()).getPath();
           tmpFileName = rootPath.substring(rootFolderFileRef.length());
         } else {
-          if (!fileCanonicalPath.startsWith(rootFolderFileRef)) {
+          if (!fileToAdd.getCanonicalFile().toPath().startsWith(rootFolderFileRef)) {
             tmpFileName = fileToAdd.getCanonicalFile().getParentFile().getName() + FILE_SEPARATOR + fileToAdd.getCanonicalFile().getName();
           } else {
             tmpFileName = fileCanonicalPath.substring(rootFolderFileRef.length());
@@ -469,11 +469,11 @@ public class FileUtils {
 
     DosFileAttributeView fileAttributeView = Files.getFileAttributeView(file, DosFileAttributeView.class, LinkOption.NOFOLLOW_LINKS);
 
-    //IntelliJ complains that fileAttributes can never be null. But apparently it can.
+    //IntelliJ complains that fileAttributeView can never be null. But apparently it can.
     //See https://github.com/srikanth-lingala/zip4j/issues/435
     //Even the javadoc of Files.getFileAttributeView says it can be null
     //noinspection ConstantConditions
-    if (fileAttributes == null) {
+    if (fileAttributeView == null) {
       return;
     }
 
